//energy and its derivatives calculations
//for NONCOLAF.CPP
//(c)V.Glazkov, March 2016

#ifndef _ENERGY
	#define _ENERGY

	#include"magvect.h"

//global variables
extern vector3d dirH;
extern double a[4][4][4][4],I[4],gamma,Hval;
extern int is_a[4][4][4][4];

//Functions declarations
	//anisotropy energy as function of magnetic vectors
	double U_A(vector3d l[]);
	//full potential energy as function of magnetic vectors and field
	double U(vector3d l[]);
	//derivative of full potential energy over i-th euler angles
	double dU_deuler(int j, double theta, double phi, double psi, int * err);
	//Hessian matrix calculations over rotation angle components
	void Hessian(void * derivspointer, double theta, double phi, double psi, int * err);
	double Hessian_component(int alpha, int beta, vector3d l[], vector3d H, int * err);
	//magnetization vector and susceptibilities
	vector3d Magnetization(vector3d l[]);
	void susceptibilities (vector3d l[],double * chipar, double * chiper);

//Functions definitions
	double U_A(vector3d l[])
		{
		int i,j,alpha,beta;
		double E=0.;

		for(i=1;i<=3;i++)
			for(j=1;j<=3;j++)
				for(alpha=1;alpha<=3;alpha++)
					for(beta=1;beta<=3;beta++)
						if(is_a[i][alpha][j][beta])
							E+=a[i][alpha][j][beta]*component(l[i],alpha)*component(l[j],beta);

		return(E);
		}

	double U(vector3d l[])
		{
		int i;
		double E=0.;

		vector3d H;

		H=Hval*dirH;

		for(i=1;i<=3;i++)
			E-=0.5*I[i]*gamma*gamma*scalar(l[i]*H,l[i]*H);

		E+=U_A(l);

		return(E);
		}

	double dU_deuler(int j, double theta, double phi, double psi, int * err)
		{
		double deriv=0.;
		int i,k,alpha,beta,code;
		vector3d H,l[4],dl[4];

		for(i=1;i<=3;i++)
			{
			l[i]=l_vec(i,theta,phi,psi,&code);
			dl[i]=dl_deuler(i,j,theta,phi,psi,&code);
			}

		H=Hval*dirH;

		for(i=1;i<=3;i++)
			deriv+=I[i]*gamma*gamma*scalar(l[i],H)*scalar(dl[i],H);

		for(i=1;i<=3;i++)
			for(k=1;k<=3;k++)
				for(alpha=1;alpha<=3;alpha++)
					for(beta=1;beta<=3;beta++)
						if(is_a[i][alpha][k][beta])
							deriv+=a[i][alpha][k][beta]*(component(dl[i],alpha)*component(l[k],beta)+component(l[i],alpha)*component(dl[k],beta));

		*err=code;
		return(deriv);
		}

	double Hessian_component(int alpha, int beta, vector3d l[], vector3d H, int * err)
		{
		double ans=0.;
		vector3d dl_alpha[4],dl_beta[4],d2l[4];
		int i,j,xi,delta,code=0;

		if(alpha==0||beta==0)
			{
			*err=code;
			return(0.);
			}
		if(alpha>3||beta>3||alpha<0||beta<0)
			{
			code=1;
			*err=code;
			return(0.);
			}

		for(i=1;i<=3;i++)
			{
			dl_alpha[i]=dl_dphi(alpha,l[i],&code);
			dl_beta[i]=dl_dphi(beta,l[i],&code);
			d2l[i]=d2l_dphi2(alpha,beta,l[i],&code);
			}


		for(i=1;i<=3;i++)
			ans+=I[i]*gamma*gamma*(scalar(dl_alpha[i],H)*scalar(dl_beta[i],H)+scalar(l[i],H)*scalar(d2l[i],H));

		for(i=1;i<=3;i++)
			for(j=1;j<=3;j++)
				for(xi=1;xi<=3;xi++)
					for(delta=1;delta<=3;delta++)
						if(is_a[i][xi][j][delta])
							ans+=a[i][xi][j][delta]*(component(dl_alpha[i],xi)*component(dl_beta[j],delta)+component(l[i],xi)*component(d2l[j],delta)+component(dl_beta[i],xi)*component(dl_alpha[j],delta)+component(d2l[i],xi)*component(l[j],delta));
		*err=code;
		return(ans);
		}

	void Hessian(void * derivspointer, double theta, double phi, double psi, int * err)
		{
		vector3d l[4],H;
		double * dblpointer;
		int i,j, code=0;

		dblpointer=(double*) derivspointer;

		H=Hval*dirH;

		for(i=1;i<=3;i++)
			l[i]=l_vec(i,theta,phi,psi,&code);

		for(i=0;i<=3;i++)
			for(j=0;j<=3;j++)
				*(dblpointer+4*i+j)=Hessian_component(i, j, l, H, &code);

		*err=code;
		}

	vector3d Magnetization(vector3d l[])
		{
		vector3d M,H;
		int i;

		H=Hval*dirH;

		M=makevector(0.,0.,0.);

		for(i=1;i<=3;i++)
			M=M+I[i]*gamma*gamma*H-I[i]*gamma*gamma*scalar(l[i],H)*l[i];

		return(M);
		}
	void susceptibilities (vector3d l[],double * chipar, double * chiper)
		{
		vector3d M,Mpar,Mper;
		int i;

		M=makevector(0.,0.,0.);

		for(i=1;i<=3;i++)
			M=M+I[i]*gamma*gamma*(dirH-scalar(l[i],dirH)*l[i]);

		Mpar=scalar(M,dirH)*dirH;
		Mper=M-Mpar;

		*chipar=mod(Mpar);
		*chiper=mod(Mper);
		}


#endif


